import path from 'path';
import { defineNuxtModule, resolvePath, addPluginTemplate } from '@nuxt/kit';

function normalizeWindowsPath(input = "") {
  if (!input.includes("\\")) {
    return input;
  }
  return input.replace(/\\/g, "/");
}

const _UNC_REGEX = /^[/][/]/;
const _UNC_DRIVE_REGEX = /^[/][/]([.]{1,2}[/])?([a-zA-Z]):[/]/;
const _IS_ABSOLUTE_RE = /^\/|^\\|^[a-zA-Z]:[/\\]/;
const sep = "/";
const delimiter = ":";
const normalize = function(path2) {
  if (path2.length === 0) {
    return ".";
  }
  path2 = normalizeWindowsPath(path2);
  const isUNCPath = path2.match(_UNC_REGEX);
  const hasUNCDrive = isUNCPath && path2.match(_UNC_DRIVE_REGEX);
  const isPathAbsolute = isAbsolute(path2);
  const trailingSeparator = path2[path2.length - 1] === "/";
  path2 = normalizeString(path2, !isPathAbsolute);
  if (path2.length === 0) {
    if (isPathAbsolute) {
      return "/";
    }
    return trailingSeparator ? "./" : ".";
  }
  if (trailingSeparator) {
    path2 += "/";
  }
  if (isUNCPath) {
    if (hasUNCDrive) {
      return `//./${path2}`;
    }
    return `//${path2}`;
  }
  return isPathAbsolute && !isAbsolute(path2) ? `/${path2}` : path2;
};
const join = function(...args) {
  if (args.length === 0) {
    return ".";
  }
  let joined;
  for (let i = 0; i < args.length; ++i) {
    const arg = args[i];
    if (arg.length > 0) {
      if (joined === void 0) {
        joined = arg;
      } else {
        joined += `/${arg}`;
      }
    }
  }
  if (joined === void 0) {
    return ".";
  }
  return normalize(joined);
};
const resolve = function(...args) {
  args = args.map((arg) => normalizeWindowsPath(arg));
  let resolvedPath = "";
  let resolvedAbsolute = false;
  for (let i = args.length - 1; i >= -1 && !resolvedAbsolute; i--) {
    const path2 = i >= 0 ? args[i] : process.cwd();
    if (path2.length === 0) {
      continue;
    }
    resolvedPath = `${path2}/${resolvedPath}`;
    resolvedAbsolute = isAbsolute(path2);
  }
  resolvedPath = normalizeString(resolvedPath, !resolvedAbsolute);
  if (resolvedAbsolute && !isAbsolute(resolvedPath)) {
    return `/${resolvedPath}`;
  }
  return resolvedPath.length > 0 ? resolvedPath : ".";
};
function normalizeString(path2, allowAboveRoot) {
  let res = "";
  let lastSegmentLength = 0;
  let lastSlash = -1;
  let dots = 0;
  let char = null;
  for (let i = 0; i <= path2.length; ++i) {
    if (i < path2.length) {
      char = path2[i];
    } else if (char === "/") {
      break;
    } else {
      char = "/";
    }
    if (char === "/") {
      if (lastSlash === i - 1 || dots === 1) ; else if (dots === 2) {
        if (res.length < 2 || lastSegmentLength !== 2 || res[res.length - 1] !== "." || res[res.length - 2] !== ".") {
          if (res.length > 2) {
            const lastSlashIndex = res.lastIndexOf("/");
            if (lastSlashIndex === -1) {
              res = "";
              lastSegmentLength = 0;
            } else {
              res = res.slice(0, lastSlashIndex);
              lastSegmentLength = res.length - 1 - res.lastIndexOf("/");
            }
            lastSlash = i;
            dots = 0;
            continue;
          } else if (res.length !== 0) {
            res = "";
            lastSegmentLength = 0;
            lastSlash = i;
            dots = 0;
            continue;
          }
        }
        if (allowAboveRoot) {
          res += res.length > 0 ? "/.." : "..";
          lastSegmentLength = 2;
        }
      } else {
        if (res.length > 0) {
          res += `/${path2.slice(lastSlash + 1, i)}`;
        } else {
          res = path2.slice(lastSlash + 1, i);
        }
        lastSegmentLength = i - lastSlash - 1;
      }
      lastSlash = i;
      dots = 0;
    } else if (char === "." && dots !== -1) {
      ++dots;
    } else {
      dots = -1;
    }
  }
  return res;
}
const isAbsolute = function(p) {
  return _IS_ABSOLUTE_RE.test(p);
};
const toNamespacedPath = function(p) {
  return normalizeWindowsPath(p);
};
const extname = function(p) {
  return path.posix.extname(normalizeWindowsPath(p));
};
const relative = function(from, to) {
  return path.posix.relative(normalizeWindowsPath(from), normalizeWindowsPath(to));
};
const dirname = function(p) {
  return path.posix.dirname(normalizeWindowsPath(p));
};
const format = function(p) {
  return normalizeWindowsPath(path.posix.format(p));
};
const basename = function(p, ext) {
  return path.posix.basename(normalizeWindowsPath(p), ext);
};
const parse = function(p) {
  return path.posix.parse(normalizeWindowsPath(p));
};

const _path = /*#__PURE__*/Object.freeze({
  __proto__: null,
  sep: sep,
  delimiter: delimiter,
  normalize: normalize,
  join: join,
  resolve: resolve,
  normalizeString: normalizeString,
  isAbsolute: isAbsolute,
  toNamespacedPath: toNamespacedPath,
  extname: extname,
  relative: relative,
  dirname: dirname,
  format: format,
  basename: basename,
  parse: parse
});

({
  ..._path
});

function genString(input, opts = {}) {
  const str = JSON.stringify(input);
  if (!opts.singleQuotes) {
    return JSON.stringify(input);
  }
  return `'${escapeString(str)}'`;
}
const NEEDS_ESCAPE_RE = /[\\'\r\n\u2028\u2029]/;
const QUOTE_NEWLINE_RE = /(['\r\n\u2028\u2029])/g;
const BACKSLASH_RE = /\\/g;
function escapeString(id) {
  if (!id.match(NEEDS_ESCAPE_RE)) {
    return id;
  }
  return id.replace(BACKSLASH_RE, "\\\\").replace(QUOTE_NEWLINE_RE, "\\$1");
}

function wrapInDelimiters(lines, indent = "", delimiters = "{}", withComma = true) {
  if (!lines.length) {
    return delimiters;
  }
  const [start, end] = delimiters;
  return `${start}
` + lines.join(withComma ? ",\n" : "\n") + `
${indent}${end}`;
}
const VALID_IDENTIFIER_RE = /^[$_]?[\w\d]*$/;
function genObjectKey(key) {
  return key.match(VALID_IDENTIFIER_RE) ? key : genString(key);
}

function genObjectFromRaw(obj, indent = "") {
  return genObjectFromRawEntries(Object.entries(obj), indent);
}
function genArrayFromRaw(array, indent = "") {
  const newIdent = indent + "  ";
  return wrapInDelimiters(array.map((i) => `${newIdent}${genRawValue(i, newIdent)}`), indent, "[]");
}
function genObjectFromRawEntries(array, indent = "") {
  const newIdent = indent + "  ";
  return wrapInDelimiters(array.map(([key, value]) => `${newIdent}${genObjectKey(key)}: ${genRawValue(value, newIdent)}`), indent, "{}");
}
function genRawValue(value, indent = "") {
  if (typeof value === "undefined") {
    return "undefined";
  }
  if (value === null) {
    return "null";
  }
  if (Array.isArray(value)) {
    return genArrayFromRaw(value, indent);
  }
  if (value && typeof value === "object") {
    return genObjectFromRaw(value, indent);
  }
  return value.toString();
}

const fileExt = /\.s[a|c]ss$/;
function getScssTransformPlugin(sassVariables) {
  const sassImportCode = ["@import 'quasar/src/css/variables.sass'", ""];
  if (typeof sassVariables === "string") {
    sassImportCode.unshift(`@import '${sassVariables}'`);
  }
  return {
    name: "vite:quasar:scss",
    enforce: "pre",
    transform(src, id) {
      const shouldTransform = id.match(fileExt)?.[0] || false;
      if (shouldTransform) {
        const prefix = shouldTransform === ".sass" ? sassImportCode.join("\n") : sassImportCode.join(";\n");
        const useIndex = Math.max(src.lastIndexOf("@use "), src.lastIndexOf("@forward "));
        const content = {
          code: src + "\n" + prefix,
          map: null
        };
        if (useIndex === -1) {
          content.code = prefix + src;
        } else {
          const newLineIndex = src.indexOf("\n", useIndex);
          if (newLineIndex !== -1) {
            const index = newLineIndex + 1;
            content.code = src.substring(0, index) + prefix + src.substring(index);
          }
        }
        return content;
      }
      return null;
    }
  };
}

const __QUASAR_VERSION__ = `'2.7.1'`;
const module = defineNuxtModule({
  meta: {
    name: "nuxt-quasar-vite",
    configKey: "quasar",
    compatibility: {
      nuxt: "^3.0.0"
    }
  },
  defaults: {
    sassVariables: true,
    css: ["quasar/src/css/index.sass"],
    plugins: ["Notify"],
    config: {
      dark: true
    }
  },
  hooks: {
    "autoImports:sources": (presets) => {
      presets.push({
        from: "quasar/src/composables",
        imports: [
          "useQuasar",
          "useDialogPluginComponent",
          "useFormChild"
        ]
      }, {
        from: "quasar/src/utils",
        imports: [
          ["clone", "qclone"],
          ["colors", "qcolors"],
          ["copyToClipboard", "qcopyToClipboard"],
          ["createMetaMixin", "qcreateMetaMixin"],
          ["createUploaderComponent", "qcreateUploaderComponent"],
          ["date", "qdate"],
          ["debounce", "qdebounce"],
          ["dom", "qdom"],
          ["event", "qevent"],
          ["exportFile", "qexportFile"],
          ["extend", "qextend"],
          ["format", "qformat"],
          ["frameDebounce", "qframeDebounce"],
          ["getCssVar", "qgetCssVar"],
          ["noop", "qnoop"],
          ["morph", "qmorph"],
          ["openURL", "qopenURL"],
          ["patterns", "qpatterns"],
          ["scroll", "qscroll"],
          ["setCssVar", "qsetCssVar"],
          ["throttle", "qthrottle"],
          ["uid", "quid"]
        ]
      });
    },
    "components:dirs": async (dirs) => {
      const source = dirname(await resolvePath("quasar/src/components")) + "/components";
      dirs.push({
        path: source,
        transpile: true,
        watch: false,
        pattern: "*/Q*.js",
        pathPrefix: false
      });
    },
    "build:before": (_, options) => {
      if (!options.transpile.includes("quasar")) {
        options.transpile.unshift("quasar");
      }
    },
    "prepare:types": ({ references }) => {
      references.unshift({ types: "quasar" });
    }
  },
  setup: (opts, nuxt) => {
    nuxt.options.css.push(...opts.css);
    nuxt.hook("vite:extendConfig", async (config, { isServer, isClient }) => {
      const define = {
        __QUASAR_VERSION__: `${__QUASAR_VERSION__}`,
        __QUASAR_SSR__: isServer,
        __QUASAR_SSR_SERVER__: isServer,
        __QUASAR_SSR_CLIENT__: isServer && isClient,
        __QUASAR_SSR_PWA__: false
      };
      Object.assign(config.define, define);
      if (opts.sassVariables) {
        const scssPath = typeof opts.sassVariables === "string" ? await resolvePath(opts.sassVariables) : true;
        const quasarScssTransform = getScssTransformPlugin(scssPath);
        config.plugins.push(quasarScssTransform);
      }
      return config;
    });
    addPluginTemplate({
      filename: "plugins/quasarVueUse.ts",
      mode: "all",
      write: true,
      getContents: () => {
        const config = genObjectFromRawEntries(Object.entries(opts.config), "		");
        const plugins = opts.plugins.join(",");
        return `import installQ from 'quasar/src/install-quasar'
import { ${plugins} } from 'quasar/src/plugins'
import lang from 'quasar/src/lang'
import iconSet from 'quasar/src/icon-set'
import * as directives from 'quasar/src/directives'

export default defineNuxtPlugin((nuxtApp) => {
  const includes = {
    directives,
    plugins: { ${plugins} },
    config: ${config},
  }
  
  nuxtApp.vueApp.use({
    version: ${__QUASAR_VERSION__},
    install(app, opts) {
      if(process.server) {
        installQ(app, {...opts, ...includes}, nuxtApp.ssrContext)
      } else {
        installQ(app, {...opts, ...includes})
      }
    },
    lang,
    iconSet
  })
})
`;
      }
    });
  }
});

export { module as default };
